"""
show_laser.py defines functions showing laser device info.
"""

from Tkinter import *
import thread
import time
import math


class show_laser (Tk):
    """
    GUI class for laser devices.
    """

    def __init__(self, player, index=0, speed=10, master=None):
        """
	create a separate window using Tkinter widgets.
	create a thread for periodic sensory data update.
	"""
	# save the configuration
	self.__player = player
	self.__device_index = index
	self.__update_speed = speed
	# create a gui
	Tk.__init__(self)
	self.title('Laser : ' + str(self.__device_index))
	self.__win = Canvas(self, bg='black', width=480, height=240)
	# set layouts of widgets
	self.__win.grid(row=0, column=0, sticky=N+E+S+W)
	self.grid()
	# draw equidistance circles
	for idx in range(30,241,30):
	    self.__win.create_arc(240-idx, 240-idx, 240+idx, 240+idx,
	                          start=0, extent=180, outline='gray40')
	# generate line items
	self.__lines = []
	for idx in range(361):
	    self.__lines.append(self.__win.create_line(240,240, 240,240, fill='yellow'))
	self.__range_count = 361
	# start an update thread
	self.__update_flag = 1
	thread.start_new_thread(self.__update_thread, ())


    def configure(self, index=0, speed=10):
        """
	change the configuration on the fly.
	"""
	self.__device_index = index
	self.__update_speed = speed


    def close(self):
        """
	close the window, and kill it self.
	"""
	self.__update_flag = 0


    def __update_thread(self):
        """
	update sensory information.
	"""
	print '<start a laser update thread>'
	while self.__update_flag:
	    try:
	        # check the current configuration
		if (self.__range_count != self.__player.laser[self.__device_index][3]):
		    range_count = self.__player.laser[self.__device_index][3]
		    if (self.__range_count < range_count):
		        for idx in range(self.__range_count, range_count):
			    # add more lines
			    self.__lines.append(self.__win.create_line(240,240, 240,240,
			                                               fill='yellow'))
		    else:
		        for idx in range(self.__range_count-1, range_count-1, -1):
			    # remove extra lines
			    self.__win.delete(self.__lines[idx])
			    del self.__lines[idx]
		    self.__range_count = range_count
		# update canvas window
		min_angle = self.__player.laser[self.__device_index][0] / 100.0
		resolution = self.__player.laser[self.__device_index][2] / 100.0
		d2r_factor = math.pi / 180
		for idx in range(self.__range_count):
		    self.__win.coords(self.__lines[idx], 240, 240,
		        240 - self.__player.laser[self.__device_index][4+idx] * 240 / 8000.0 *
			      math.sin((min_angle+idx*resolution) * d2r_factor),
		        240 - self.__player.laser[self.__device_index][4+idx] * 240 / 8000.0 *
			      math.cos((min_angle+idx*resolution) * d2r_factor))
		# slow down
		time.sleep(1.0 / self.__update_speed)
	    except KeyError:
	        print '<laser:%d is not enabled yet>' % self.__device_index
		self.close()
	    except AttributeError:
		print '<invalid player object>'
	        self.close()
	    except TclError:
		self.close()
	print '<the laser update thread finishes>'

